/***********************************************************************************
NAV DB Copy - Copy companies between databases      
Copyright (c) 2014 Ingmar Stieger

Usage:

1) Create an exact copy of the source database, all tables must be identical. Importing the
   objects as FOB is fine. This will be the destination database
2) Connect to the new database using the Windows Client and create a new Company. It must have
   exactly the same name as the Company in the source database
3) Open this script in a new Query Window in SQL Server Management Studio
4) Set the name of the destination database below ("USE DATABASENAME...")
5) Scroll down and set the name of the source database. You may also specify a linked server if
   the source database is on a remote SQL Server
6) Set the name of the company to copy
7) Optional: Specify all the DataPerCompany=No tables that you want to copy along by adding one 
   "EXECUTE NAVCopyTable..." per table. Examples can be found below
8) Run the script to start the copy

License and disclaimer

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
************************************************************************************/

/* Set the destination database here: */
USE DATABASENAME
GO


IF OBJECT_ID('NAVFieldList', 'P') IS NOT NULL
    DROP PROCEDURE NAVFieldList;
GO

CREATE PROCEDURE NAVFieldList
	@table nvarchar(MAX),
	@list nvarchar(MAX) OUTPUT
AS  
	DECLARE @name nvarchar(MAX)
	DECLARE @fieldquery AS NVARCHAR(MAX)
	DECLARE @field_cursor CURSOR 

	SET @fieldquery = N'
		SET @field_cursor = CURSOR FAST_FORWARD FOR 
			SELECT name FROM sys.columns WHERE object_id = OBJECT_ID(''' + @table + ''') and Name != N''timestamp'' 
		OPEN @field_cursor;'
	EXECUTE sp_executesql @fieldquery, N'@field_cursor CURSOR OUTPUT', @field_cursor = @field_cursor OUTPUT

	SET @list = N''
	FETCH NEXT FROM @field_cursor INTO @name
	WHILE @@FETCH_STATUS = 0
	BEGIN
		IF @list != N''
			SET @list += N','
		SET @list += QUOTENAME(RTRIM(@name))
		FETCH NEXT FROM @field_cursor INTO @name
	END
	CLOSE @field_cursor
	DEALLOCATE @field_cursor
	
	RETURN
GO

IF OBJECT_ID('NAVCopyTable', 'P') IS NOT NULL
    DROP PROCEDURE NAVCopyTable;
GO

CREATE PROCEDURE NAVCopyTable
	@sourceDB nvarchar(MAX),	
	@table nvarchar(MAX)
AS  
	DECLARE @fields nvarchar(MAX)
	DECLARE @execquery AS NVARCHAR(MAX)

	PRINT @table
	EXECUTE NAVFieldList @table, @fields OUTPUT;
	
	SET @execquery = 'DELETE FROM ' + QUOTENAME(@table)
	EXECUTE sp_executesql @execquery
	
	SET @execquery = ''
	IF OBJECTPROPERTY(OBJECT_ID(@table), 'TableHasIdentity') = 1
		SET @execquery = 'SET IDENTITY_INSERT ' + QUOTENAME(@table) + 'ON; ' 
		
	SET @execquery = @execquery + 
	    'INSERT INTO ' + QUOTENAME(@table) + 
		' (' + @fields + ') SELECT ' + @fields + ' FROM ' + @sourceDB + '.dbo.' + QUOTENAME(@table)
	EXECUTE sp_executesql @execquery
GO

/* Set the source database here

   Example for specifying a linked server to copy from:
   DECLARE @sourceDB nvarchar(MAX) = QUOTENAME(N'SERVERNAME') + '.' + QUOTENAME(N'DATABASENAME')
*/

DECLARE @sourceDB nvarchar(MAX) = QUOTENAME(N'DATABASENAME')

/* Set the company name here: */
DECLARE @company nvarchar(MAX) = N'CRONUS AG'

DECLARE @table nvarchar(MAX)
DECLARE @table_count int = 0
DECLARE @tablequery AS NVARCHAR(MAX)
DECLARE @table_cursor CURSOR

/* DataPerCompany = No Tables (optional - comment if not needed) */
/*
EXECUTE NAVCopyTable @sourceDB, N'Chart'
EXECUTE NAVCopyTable @sourceDB, N'Client Add-in'
EXECUTE NAVCopyTable @sourceDB, N'Permission'
EXECUTE NAVCopyTable @sourceDB, N'Permission Set'
EXECUTE NAVCopyTable @sourceDB, N'Profile'
EXECUTE NAVCopyTable @sourceDB, N'Profile Metadata'
SET @table_count += 6
*/

/* DataPerCompany = Yes Tables */
SET @tablequery = N'
	SET @table_cursor = CURSOR FAST_FORWARD FOR 
		SELECT TABLE_NAME FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_TYPE = N''BASE TABLE'' AND TABLE_NAME LIKE N''' + @company + N'$%'';
	OPEN @table_cursor;'
EXECUTE sp_executesql @tablequery, N'@table_cursor CURSOR OUTPUT', @table_cursor = @table_cursor OUTPUT

FETCH NEXT FROM @table_cursor INTO @table
WHILE @@FETCH_STATUS = 0
BEGIN
	SET @table_count += 1
    EXECUTE NAVCopyTable @sourceDB, @table
	FETCH NEXT FROM @table_cursor INTO @table
END
CLOSE @table_cursor
DEALLOCATE @table_cursor

RAISERROR(N'Execution finished. %d tables were copied.', 0, 1, @table_count)
GO

DROP PROCEDURE NAVFieldList;
DROP PROCEDURE NAVCopyTable;
GO
